/* Copyright (c) 2016 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** 
* This example shows how to operate application timers when softdevice is not used. The
* application timers use RTC1 in the background, which in turn needs 32kHz low frequency
* clock to operate. This example uses the internal RC as low frequency clock. The internal
* RC needs to be periodically calibrated to obtain acceptable accuracy, i.e. 250ppm. The
* nrf_drv_clock driver is used to manually start the low frequency clock and also to 
* calibrate the internal RC low frequency clock.
*
* Peripherals used: RTC, Clock
* Compatibility: nRF51 third revision, SDK 11.0.0, nRF51-DK
* Softdevice used: None
* Operation:
* LED1 toggles to indicate application timer interrupt
* LED3 toggles to indicate calibration complete
*/

#include <stdbool.h>
#include <stdint.h>

#include "nrf.h"
#include "nrf_gpio.h"
#include "nordic_common.h"
#include "nrf_delay.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_util_platform.h"
#include "boards.h"
#include "bsp.h"
#include "nrf_drv_clock.h"

#define APP_TIMER_PRESCALER             0                                          /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE         4                                          /**< Size of timer operation queues. */
#define TIMER1_INTERVAL                 APP_TIMER_TICKS(20, APP_TIMER_PRESCALER)   /**< Defines the interval between consecutive app timer interrupts in milliseconds. */
#define LFCLK_CALIBRATION_INTERVAL      8                                          /**< Defines the LFCLK RC calibration interval in 0.25 millisecond steps. */

APP_TIMER_DEF(m_timer1_id); 

static void timer1_handler(void * p_context)
{
    LEDS_INVERT(BSP_LED_0_MASK);                                                                //Indicate execution of timer1 interrupt handler on LED1
}

static void lfclk_rc_calibration_handler(nrf_drv_clock_evt_type_t event)
{
		LEDS_INVERT(BSP_LED_2_MASK);                                                                //Indicate execution of calibration interrupt handler on LED3
		nrf_drv_clock_calibration_start(LFCLK_CALIBRATION_INTERVAL, lfclk_rc_calibration_handler);  //Schedule lfclk calibration after LFCLK_CALIBRATION_INTERVAL
}

static void timers_init(void)
{
    uint32_t err_code;

    // Initialize timer module.
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);

    // Create timers.
    err_code = app_timer_create(&m_timer1_id,
                                APP_TIMER_MODE_REPEATED,
                                timer1_handler);
    APP_ERROR_CHECK(err_code);
}

static void application_timers_start(void)
{
    uint32_t err_code;

    // Start application timers.
    err_code = app_timer_start(m_timer1_id, TIMER1_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
}

static void clock_init(void)
{
    uint32_t err_code;
	
		err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);
		nrf_drv_clock_lfclk_request(NULL);
		while(!nrf_drv_clock_lfclk_is_running());                                    //Wait until the lfclk is running
		nrf_drv_clock_calibration_start(0, lfclk_rc_calibration_handler);            //Start lfclk RC calibration to improve accuracy to 250ppm
//		while(is_calibrating) {	nrf_drv_clock_is_calibrating(&is_calibrating); }   //this wait does not work if calibration is triggered again in the lfclk_rc_calibration_handler calibration handler. The reason is that as soon as the nrf_drv_clock_is_calibrating function is called in the calibration handler, even when the calibration timer is non-zero, the calibration state is not idle and this check will fail.
}

/**
 * @brief Function for application main entry.
 */
int main(void)
{
		LEDS_CONFIGURE(LEDS_MASK);
		LEDS_OFF(LEDS_MASK);
	
		clock_init();
		timers_init();
		application_timers_start();

    while (true)
    {
        // Do nothing.
    }
}
/** @} */
